/*
 *    **************************************************
 *    *                                                *
 *    *                   BIGCALC.C                    *
 *    *                                                *
 *    *          Extended Precision Calculator         *
 *    *                                                *
 *    *                  Main Module                   *
 *    *                                                *
 *    *              Version 2.11 03-27-88             *
 *    *                                                *
 *    *              Judson D. McClendon               *
 *    *              844 Sun Valley Road               *
 *    *              Birmingham, AL 35215              *
 *    *                 205-853-8440                   *
 *    *            Compuserve [74415,1003]             *
 *    *                                                *
 *    **************************************************
 */



/*
 *    **************************************************
 *    *                                                *
 *    *                   Includes                     *
 *    *                                                *
 *    **************************************************
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bigcalc.h"



/*
 *    **************************************************
 *    *                                                *
 *    *           Global Variable Definition           *
 *    *                                                *
 *    **************************************************
 */

NORMTYPE
   stack[4],               /* Stack registers  */
   reg[10],                /* Memory registers */
   lastx;                  /* Last X register  */

WORKTYPE
   work[5];                /* Work registers   */

union {
   WORKTYPE work;          /* Temp work reg    */
   NORMTYPE norm;          /* Temp norm reg    */
   } temp;

int
   normprec = 0,           /* Normal precision  */
   compprec = 0,           /* Compute precision */
   workprec = 0,           /* Work precision    */
   chr = 0,                /* Input Character   */
   ppos = 0,               /* Current print col */
   groupsize = 0,          /* Digit group size  */
   entrysignrow;           /* Row for X entry   */

long
   minfloatprn,            /* Min exp for float */
   maxfloatprn;            /* Max exp for float */

FILE
   *printfile;             /* Print file handle */

char
   stackname[4]            /* Stack register names */
      = {'X','Y','Z','T'},
   printid[15]             /* Print file name      */
      = "Printer";

BOOLEAN
   stacklift = TRUE,       /* Lift stack for new X if TRUE */
   scinotation = FALSE,    /* Force sci notation if TRUE   */
   charpresent = FALSE,    /* Character present if TRUE    */
   menucleared = TRUE,     /* Screen menu cleared if TRUE  */
   printtoscreen = FALSE;  /* Print to screen if TRUE      */



/*
 *    **************************************************
 *    *                                                *
 *    *                     Main                       *
 *    *                                                *
 *    **************************************************
 */

void main(int argc,char *argv[])

{

   Initialize(argc, argv[1]);

   chr = GetPrompt();

   while (chr != ESCAPE) {

      switch (chr) {

         case (ADD):
            Add();               /* Add Y + X */
            break;

         case (SUBTRACT):
            Subtract();          /* Subtract Y - X */
            break;

         case (MULTIPLY):
            Multiply();          /* Multiply Y * X */
            break;

         case (DIVIDE):
            Divide();            /* Divide Y / X */
            break;

         case (HELP):
            HelpScreen();        /* Help Screen */
            break;

         case (ABS):
            AbsoluteValue();     /* Absolute Value X */
            break;

         case (SQUAREROOT):
            SquareRoot();        /* Square Root X */
            break;

         case (SQUARE):
            Square();            /* Square X */
            break;

         case (RECIPROCAL):
            Reciprocal();        /* Reciprocal X */
            break;

         case (FACTORIAL):
            Factorial();         /* Factorial X */
            break;

         case (INTEGER):
            IntegerPart();       /* Integer Part X */
            break;

         case (FRACTION):
            FractionPart();      /* Fraction Part X */
            break;

         case (RECALLPI):
            RecallPi();          /* Recall pi */
            break;

         case (RECALLE):
            RecallE();           /* Recall e */
            break;

         case (LASTX):
            RecallLastX();       /* Recall Last X */
            break;

         case (CHGSIGN):
            ChangeSign();        /* Change sign X */
            break;

         case (PRINTDISK):
            PrintDisk();         /* Toggle Print to Disk */
            break;

         case (GROUPSIZE):
            GroupSize();         /* Toggle Group Size (3/5) */
            break;

         case (VIEWREG):
            ViewReg();           /* View Register on Screen */
            break;

         case ('0'):
         case ('1'):
         case ('2'):
         case ('3'):
         case ('4'):
         case ('5'):
         case ('6'):
         case ('7'):
         case ('8'):
         case ('9'):
         case ('.'):
         case ('E'):
            AcceptX();           /* Accept new X from keyboard with first */
            break;               /*  character passed to AcceptX routine */

         case (CLEARX):
            ClearX();            /* Clear X to zero */
            break;

         case (ENTER):
            Enter();             /* Push up stack */
            break;

         case (SCINOT):
            SciNotation();       /* Use Scientific Notation */
            break;

         case (CLEAR):
            Clear();             /* Clear (prompt for what) */
            break;

         case (PRINT):
            Print();             /* Print on printer (prompt for what) */
            break;

         case (STOREX):
            StoreX();            /* Store X in register (prompt for which) */
            break;

         case (RECALLREG):
            RecallReg();         /* Recall register to X (prompt for which) */
            break;

         case (XCHGXY1):
         case (XCHGXY2):
            ExchangeXY();        /* Exchange X and Y */
            break;

         case (XCHGXREG):
            ExchangeXReg();      /* Exchange X and Reg (prompt for which) */
            break;

         case (ROLLDOWN):
            RollDown();          /* Roll stack down */
            break;

         case (ROLLUP):
            RollUp();            /* Roll stack up */
            break;

         default:
            Beep();              /* Unknown key */

         }  /* end switch */

      flushall();

      chr = GetPrompt();

      }  /* while */


   if (printid[0] == 'B')        /* Close BIGCALC.PRN if open */
      fclose(printfile);

   CurPos(20, 1);
   EraEop();
   exit(0);

}





/*
 *    **************************************************
 *    *                                                *
 *    *              Initialize Variables              *
 *    *                                                *
 *    **************************************************
 */

void Initialize(int argc, char *argv)

{

   if (argc > 1)
      while (*argv) {               /* convert *argv to number */
         if (isdigit(*argv)) {
            normprec = (normprec * 10) + (*argv - '0');
            if (normprec <= NORMPREC)
               argv++;
               continue;
            }
         normprec = 0;
         break;
         }

   if ( (normprec < MINPREC)
         ||
        (normprec > NORMPREC) )
      normprec = DEFAULTPREC;

   compprec = normprec + 5;
   workprec = (compprec * 2) + 5;

   ClearStack(0, 3);             /* Initialize work areas */
   ClearReg(0, 9);
   ClearWork(0, 2);
   lastx = stack[0];
   printfile = stdprn;
                                 /* Allow full screen entry for big numbers */
   if (normprec > SIZEOFSMALL)
      entrysignrow = ENTRYSIGNROWBIG;
   else
      entrysignrow = ENTRYSIGNROWSMALL;

   GroupSize();                  /* Toggle group size to 5 & set xxxfloatprn */

   InitialScreen();              /* Welcome screen */

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Process Routines                *
 *    *                                                *
 *    **************************************************
 */



/*
 *    **************************************************
 *    *                                                *
 *    *                 Add X = Y + X                  *
 *    *                                                *
 *    **************************************************
 */

void Add(void)

{

   Message("Adding...");

   MoveStackWork(0, 0);
   MoveStackWork(1, 1);

   if (ExtendedAdd() ) {
      lastx = stack[0];
      DropStack();
      MoveWorkStack(2, 0);
      WriteStack(0, 2);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *              Subtract X = Y - X                *
 *    *                                                *
 *    **************************************************
 */

void Subtract(void)

{

   Message("Subtracting...");

   MoveStackWork(0, 0);
   MoveStackWork(1, 1);

   if (ExtendedSubtract() ) {
      lastx = stack[0];
      DropStack();
      MoveWorkStack(2, 0);
      WriteStack(0, 2);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *              Multiply X = Y * X                *
 *    *                                                *
 *    **************************************************
 */

void Multiply(void)

{

   MessageEsc("Multiplying");

   MoveStackWork(0, 0);
   MoveStackWork(1, 1);

   if (ExtendedMultiply() ) {
      lastx = stack[0];
      DropStack();
      MoveWorkStack(2, 0);
      WriteStack(0, 2);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *               Divide X = Y / X                 *
 *    *                                                *
 *    **************************************************
 */

void Divide(void)

{

   MessageEsc("Dividing");

   MoveStackWork(0, 0);
   MoveStackWork(1, 1);

   if (ExtendedDivide() ) {
      lastx = stack[0];
      DropStack();
      MoveWorkStack(2, 0);
      WriteStack(0, 2);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *              Absolute Value of X               *
 *    *                                                *
 *    **************************************************
 */

void AbsoluteValue(void)

{

   if (stack[0].sign == '-') {
      lastx = stack[0];
      stack[0].sign = '+';
      WriteStack(0, 0);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *               Square Root of X                 *
 *    *                                                *
 *    **************************************************
 */

void SquareRoot(void)

{

   MessageEsc("Computing Square Root");

   MoveStackWork(0, 3);
   if (ExtendedSquareRoot() ) {
      lastx = stack[0];
      MoveWorkStack(2, 0);
      WriteStack(0, 0);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *                 Square of X                    *
 *    *                                                *
 *    **************************************************
 */

void Square(void)

{

   MessageEsc("Computing Square");

   MoveStackWork(0, 0);
   MoveStackWork(0, 1);

   if (ExtendedMultiply() ) {
      lastx = stack[0];
      MoveWorkStack(2, 0);
      WriteStack(0, 0);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Reciprocal of X                 *
 *    *                                                *
 *    **************************************************
 */

void Reciprocal(void)

{

   MessageEsc("Computing Reciprocal");

   MoveStackWork(0, 0);

   if (ExtendedReciprocal() ) {
      lastx = stack[0];
      MoveWorkStack(2, 0);
      WriteStack(0, 0);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Factorial of X                  *
 *    *                                                *
 *    **************************************************
 */

void Factorial(void)

{

   MessageEsc("Computing Factorial");

   MoveStackWork(0, 3);

   if (ExtendedFactorial() ) {
      lastx = stack[0];
      MoveWorkStack(2, 0);
      WriteStack(0, 0);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *           Extract Integer Part of X            *
 *    *                                                *
 *    **************************************************
 */

void IntegerPart(void)

{

   MoveStackWork(0, 0);

   if (ExtendedIntegerPart() ) {
      lastx = stack[0];
      MoveWorkStack(0, 0);
      WriteStack(0, 0);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *          Extract Fraction Part of X            *
 *    *                                                *
 *    **************************************************
 */

void FractionPart(void)

{

   MoveStackWork(0, 0);

   if (ExtendedFractionPart() ) {
      lastx = stack[0];
      MoveWorkStack(0, 0);
      WriteStack(0, 0);
      stacklift = TRUE;
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Recall pi to X                  *
 *    *                                                *
 *    **************************************************
 */

void RecallPi(void)

{

   if (stacklift) {
      PushStack();
      WriteStack(1, 3);
      }

   ExtendedRecallPi();
   MoveWorkStack(0, 0);
   WriteStack(0, 0);
   stacklift = TRUE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Recall e to X                   *
 *    *                                                *
 *    **************************************************
 */

void RecallE(void)

{

   if (stacklift) {
      PushStack();
      WriteStack(1, 3);
      }

   ExtendedRecallE();
   MoveWorkStack(0, 0);
   WriteStack(0, 0);
   stacklift = TRUE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Recall Last X                   *
 *    *                                                *
 *    **************************************************
 */

void RecallLastX(void)

{

   if (stacklift) {
      PushStack();
      WriteStack(1, 3);
      }

   stack[0] = lastx;
   WriteStack(0, 0);
   stacklift = TRUE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *               Change Sign of X                 *
 *    *                                                *
 *    **************************************************
 */

void ChangeSign(void)

{

   if (stack[0].digits) {        /* Only if X non zero */
      FlipSign(&stack[0].sign);
      CurPos(XSIGNROW, SIGNDISPCOL);
      if (stack[0].sign == '-')
         WChar('-');
      else
         WChar(' ');
      }

   stacklift = TRUE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *              Toggle Print to Disk              *
 *    *                                                *
 *    **************************************************
 */

void PrintDisk(void)

{

   if (printid[0] == 'B') {
      fclose(printfile);                  /* BIGCALC.PRN open, close */
      strcpy(printid, "Printer");         /*  and switch to printer  */
      printfile = stdprn;
      }
   else {                                 /* Open BIGCALC.PRN */
      strcpy(printid, "BIGCALC.PRN");
      if ((printfile = fopen(printid, "ab")) == NULL) {
         strcpy(printid, "Printer");
         printfile = stdprn;
         MessageWait("** Cannot open disk file **");
         }
      }

   Heading2();

}




/*
 *    **************************************************
 *    *                                                *
 *    *             Toggle Group Size (3/5)            *
 *    *                                                *
 *    **************************************************
 */

void GroupSize(void)

{

   static BOOLEAN flag;

   if (groupsize)             /* First time thru groupsize == 0 */
      flag = TRUE;
   else
      flag = FALSE;

   if (groupsize == 5) {
      groupsize = 3;
      minfloatprn = (long)normprec - 1077L;
      maxfloatprn = 1080L;
      }
   else {
      groupsize = 5;
      minfloatprn = (long)normprec - 1195L;
      maxfloatprn = 1200L;
      }

   if (flag)
      Heading2();

}




/*
 *    **************************************************
 *    *                                                *
 *    *             View Register on Screen            *
 *    *                                                *
 *    **************************************************
 */

void ViewReg(void)

{

   Message("Press to view (X,Y,Z,T, 0-9, Esc=Exit):");

   while ((chr = GetChar()) != ESCAPE) {

      switch (chr) {

         case ('X'):
            DisplayStack(0);
            return;

         case ('Y'):
            DisplayStack(1);
            return;

         case ('Z'):
            DisplayStack(2);
            return;

         case ('T'):
            DisplayStack(3);
            return;

         case ('0'):
         case ('1'):
         case ('2'):
         case ('3'):
         case ('4'):
         case ('5'):
         case ('6'):
         case ('7'):
         case ('8'):
         case ('9'):
            DisplayReg(chr - '0');
            return;

         default:
            Beep();

         }  /* switch */

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *               Accept X from KBD                *
 *    *                                                *
 *    **************************************************
 */

void AcceptX(void)

{

   CurPos(entrysignrow, 1);      /* Clear area to enter new X */
   EraEop();
   menucleared = TRUE;
   WriteAt(entrysignrow, 1, "X:");
   Message("Entering X: S=ChgSign, E=Exp, BakSpc=Backup, Other=Complete,"
           " ESC=Exit");

   if (normprec > SIZEOFSMALL) {

      CurPos(entrysignrow - 1, 1);  /* Big numbers, use full screen */
      EraEol();
      WriteAt(entrysignrow - 1, 1, "========================="
                                   "  E N T E R I N G   X  "
                                   "=========================");
      if (ExtendedGetX() ) {
          if (stacklift)
             PushStack();
         MoveWorkStack(0, 0);
         stacklift = TRUE;
         }
      WorkScreen();
      }

   else {

      if (stacklift) {              /* Small numbers, use bottom of screen */
         temp.norm = stack[3];
         PushStack();
         WriteStack(1, 3);
         }
      if (ExtendedGetX() ) {
         MoveWorkStack(0, 0);
         stacklift = TRUE;
         }
      else
         if (stacklift) {
            DropStack();
            stack[3] = temp.norm;
            WriteStack(1, 3);
         }
      CurPos(entrysignrow, SIGNDISPCOL);
      EraEop();
      WriteStack(0, 0);
      }

}




/*
 *    **************************************************
 *    *                                                *
 *    *                    Clear X                     *
 *    *                                                *
 *    **************************************************
 */

void ClearX(void)

{

   ClearStack(0, 0);
   WriteStack(0, 0);
   stacklift = FALSE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *                    Enter                       *
 *    *                                                *
 *    **************************************************
 */

void Enter(void)

{

   PushStack();
   WriteStack(1, 3);
   stacklift = FALSE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *          Toggle Scientific Notation            *
 *    *                                                *
 *    **************************************************
 */

void SciNotation(void)

{

   if (scinotation)
      scinotation = FALSE;
   else
      scinotation = TRUE;

   WriteReg(0, 9);
   WriteStack(0, 3);

}




/*
 *    **************************************************
 *    *                                                *
 *    *        Clear (S, X,Y,Z,T, R, 0-9, All)         *
 *    *                                                *
 *    **************************************************
 */

void Clear(void)

{

   Message("Press to clear (S, X,Y,Z,T, R, 0-9, A=All, Esc=Exit):");

   while ((chr = GetChar()) != ESCAPE) {

      switch (chr) {

         case ('S'):
            ClearStack(0, 3);
            WriteStack(0, 3);
            return;

         case ('X'):
            ClearStack(0, 0);
            WriteStack(0, 0);
            return;

         case ('Y'):
            ClearStack(1, 1);
            WriteStack(1, 1);
            return;

         case ('Z'):
            ClearStack(2, 2);
            WriteStack(2, 2);
            return;

         case ('T'):
            ClearStack(3, 3);
            WriteStack(3, 3);
            return;

         case ('R'):
            ClearReg(0, 9);
            WriteReg(0, 9);
            return;

         case ('0'):
         case ('1'):
         case ('2'):
         case ('3'):
         case ('4'):
         case ('5'):
         case ('6'):
         case ('7'):
         case ('8'):
         case ('9'):
            ClearReg((chr - '0'), (chr - '0'));
            WriteReg((chr - '0'), (chr - '0'));
            return;

         case ('A'):
            ClearReg(0, 9);
            ClearStack(0, 3);
            WriteReg(0, 9);
            WriteStack(0, 3);
            return;

         default:
            Beep();

         }  /* switch */

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *        Print (S, X,Y,Z,T, R, 0-9, All)         *
 *    *                                                *
 *    **************************************************
 */

void Print(void)

{

   Message("Press to print (S, X,Y,Z,T, R, 0-9, A=All, P=Page Eject,"
           " Esc=Exit):");

   while ((chr = GetChar()) != ESCAPE) {

      switch (chr) {

         case ('S'):
            PrintStack(0, 3);
            return;

         case ('X'):
            PrintStack(0, 0);
            return;

         case ('Y'):
            PrintStack(1, 1);
            return;

         case ('Z'):
            PrintStack(2, 2);
            return;

         case ('T'):
            PrintStack(3, 3);
            return;

         case ('R'):
            PrintReg(0, 9);
            return;

         case ('0'):
         case ('1'):
         case ('2'):
         case ('3'):
         case ('4'):
         case ('5'):
         case ('6'):
         case ('7'):
         case ('8'):
         case ('9'):
            PrintReg((chr - '0'), (chr - '0'));
            return;

         case ('A'):
            PrintReg(0, 9);
            PrintStack(0, 3);
            return;

         case ('P'):
            NewPage();
            return;

         default:
            Beep();

         }  /* switch */

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *              Store X in Register               *
 *    *                                                *
 *    *             Allows + - * / as well             *
 *    *                                                *
 *    **************************************************
 */

void StoreX(void)

{

   Message("Store X to: Press Reg (0-9) or Operation (+,-,*,/)"
           " then Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         chr = chr - '0';
         reg[chr] = stack[0];
         WriteReg(chr, chr);
         stacklift = TRUE;
         return;
         }

      switch (chr) {

         case (ADD):
            AddXReg();
            return;

         case (SUBTRACT):
            SubtractXReg();
            return;

         case (MULTIPLY):
            MultiplyXReg();
            return;

         case (DIVIDE):
            DivideXReg();
            return;

         default:
            Beep();

         }  /* switch */

      } /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *               Add X to Register                *
 *    *                                                *
 *    **************************************************
 */

void AddXReg(void)

{

   Message("Add X to: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         Message("Adding...");

         chr = chr - '0';
         MoveStackWork(0, 0);
         MoveRegWork(chr, 1);

         if (ExtendedAdd() ) {
            MoveWorkReg(2, chr);
            WriteReg(chr, chr);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *           Subtract X from Register             *
 *    *                                                *
 *    **************************************************
 */

void SubtractXReg(void)

{

   Message("Subtract X from: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         Message("Subtracting...");

         chr -= '0';
         MoveStackWork(0, 0);
         MoveRegWork(chr, 1);

         if (ExtendedSubtract() ) {
            MoveWorkReg(2, chr);
            WriteReg(chr, chr);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *            Multiply X times Register           *
 *    *                                                *
 *    **************************************************
 */

void MultiplyXReg(void)

{

   Message("Multiply X times: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         MessageEsc("Multiplying");

         chr -= '0';
         MoveStackWork(0, 0);
         MoveRegWork(chr, 1);

         if (ExtendedMultiply() ) {
            MoveWorkReg(2, chr);
            WriteReg(chr, chr);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *             Divide X into Register             *
 *    *                                                *
 *    **************************************************
 */

void DivideXReg(void)

{

   Message("Divide X into: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         MessageEsc("Dividing");

         chr -= '0';
         MoveStackWork(0, 0);
         MoveRegWork(chr, 1);

         if (ExtendedDivide() ) {
            MoveWorkReg(2, chr);
            WriteReg(chr, chr);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *             Recall Register to X               *
 *    *                                                *
 *    **************************************************
 */

void RecallReg(void)

{

   Message("Recall to X: Press Reg (0-9) or Operation (+,-,*,/)"
           " then Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         chr -= '0';
         if (stacklift)
            PushStack();
         stack[0] = reg[chr];
         if (stacklift)
            WriteStack(0, 3);
         else
            WriteStack(0, 0);
         stacklift = TRUE;
         return;
         }

      switch (chr) {

         case (ADD):
            AddRegX();
            return;

         case (SUBTRACT):
            SubtractRegX();
            return;

         case (MULTIPLY):
            MultiplyRegX();
            return;

         case (DIVIDE):
            DivideRegX();
            return;

         default:
            Beep();

         }  /* switch */

      } /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *               Add Register to X                *
 *    *                                                *
 *    **************************************************
 */

void AddRegX(void)

{

   Message("Add to X: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         Message("Adding...");

         chr -= '0';
         MoveRegWork(chr, 0);
         MoveStackWork(0, 1);

         if (ExtendedAdd() ) {
            lastx = stack[0];
            MoveWorkStack(2, 0);
            WriteStack(0, 0);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *           Subtract Register from X             *
 *    *                                                *
 *    **************************************************
 */

void SubtractRegX(void)

{

   Message("Subtract from X: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         Message("Subtracting...");

         chr -= '0';
         MoveRegWork(chr, 0);
         MoveStackWork(0, 1);

         if (ExtendedSubtract() ) {
            lastx = stack[0];
            MoveWorkStack(2, 0);
            WriteStack(0, 0);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *            Multiply Register times X           *
 *    *                                                *
 *    **************************************************
 */

void MultiplyRegX(void)

{

   Message("Multiply times X: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         MessageEsc("Multiplying");

         chr -= '0';
         MoveRegWork(chr, 0);
         MoveStackWork(0, 1);

         if (ExtendedMultiply() ) {
            lastx = stack[0];
            MoveWorkStack(2, 0);
            WriteStack(0, 0);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      } /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *             Divide Register into X             *
 *    *                                                *
 *    **************************************************
 */

void DivideRegX(void)

{

   Message("Divide into X: Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         MessageEsc("Dividing");

         chr -= '0';
         MoveRegWork(chr, 0);
         MoveStackWork(0, 1);

         if (ExtendedDivide() ) {
            lastx = stack[0];
            MoveWorkStack(2, 0);
            WriteStack(0, 0);
            stacklift = TRUE;
            return;
            }
         return;
         }
      else
         Beep();

      }  /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Exchange X & Y                  *
 *    *                                                *
 *    **************************************************
 */

void ExchangeXY(void)

{

   temp.norm = stack[0];
   stack[0] = stack[1];
   stack[1] = temp.norm;

   WriteStack(0, 1);
   stacklift = TRUE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *             Exchange X & Register              *
 *    *                                                *
 *    **************************************************
 */

void ExchangeXReg(void)

{

   Message("Exchange X with: Press Reg (0-9) or Esc:");

   while ((chr = GetChar()) != ESCAPE) {

      if (isdigit(chr) ) {
         chr = chr - '0';
         temp.norm = stack[0];
         stack[0] = reg[chr];
         reg[chr] = temp.norm;
         WriteReg(chr, chr);
         WriteStack(0, 0);
         stacklift = TRUE;
         return;
         }

      } /* while */

}




/*
 *    **************************************************
 *    *                                                *
 *    *                Roll Stack Down                 *
 *    *                                                *
 *    **************************************************
 */

void RollDown(void)

{

   temp.norm = stack[0];
   DropStack();
   stack[3] = temp.norm;

   WriteStack(0, 3);
   stacklift = TRUE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *                 Roll Stack Up                  *
 *    *                                                *
 *    **************************************************
 */

void RollUp(void)

{

   temp.norm = stack[3];
   PushStack();
   stack[0] = temp.norm;

   WriteStack(0, 3);
   stacklift = TRUE;

}




/*
 *    **************************************************
 *    *                                                *
 *    *                  Push Stack                    *
 *    *                                                *
 *    **************************************************
 */

void PushStack(void)

{

   static int s;

   for (s = 3; s >= 1; s--)
      stack[s] = stack[s - 1];

}




/*
 *    **************************************************
 *    *                                                *
 *    *                  Drop Stack                    *
 *    *                                                *
 *    **************************************************
 */

void DropStack(void)

{

   static int s;

   for (s = 0; s <= 2; s++)
      stack[s] = stack[s + 1];

}

